$(document).ready(function () {
  const SUCCESS_CONECTION_MESSAGE = M.util.get_string(
    "success_connection_message",
    "block_bgwidget"
  );
  const CONECTION_FAILURE_MESSAGE = M.util.get_string(
    "connection_failure_message",
    "block_bgwidget"
  );
  const SEND_MESSAGE_FAILURE_MESSAGE = M.util.get_string(
    "send_message_failure_message",
    "block_bgwidget"
  );
  const CHANNEL_ACRONYM = "MO";

  var tokenKey = "chatToken_" + bot_id;
  var messagesKey = "chatMessages_" + bot_id;
  var token = sessionStorage.getItem(tokenKey) || "";
  var messages = JSON.parse(sessionStorage.getItem(messagesKey)) || [];

  // Función principal para iniciar el chat
  function initialize() {
    loadPreviousMessages();
    if (!token) {
      initializeChat();
    }
    setupEventListeners();
  }

  // Mostrar mensajes anteriores si existen
  function loadPreviousMessages() {
    if (messages.length > 0) {
      messages.forEach(function (message) {
        addMessage(message.sender, message.content, message.isUser);
      });
    }
  }

  // Configurar los event listeners
  function setupEventListeners() {
    $("#chat-send").on("click", function () {
      handleSendMessage();
    });

    $("#chat-input").on("keypress", function (e) {
      if (e.which === 13) {
        handleSendMessage();
      }
    });

    $("#chat-reset").on("click", function () {
      resetChat();
    });
  }

  // Iniciar el chat conectando con la API
  function initializeChat() {
    var data = {
      bot_id: bot_id,
      env: env,
      channel: CHANNEL_ACRONYM,
      username: stringToHex(user_name),
    };

    sendPostRequest(
      api_base_url + "/secure/api/connect",
      data,
      handleChatConnectSuccess,
      function () {
        showError(CONECTION_FAILURE_MESSAGE);
      }
    );
  }

  // Manejar la conexión exitosa al chat
  function handleChatConnectSuccess(connectResponse) {
    token = connectResponse.data["token"];
    sessionStorage.setItem(tokenKey, token);
    var initialMessage =
      connectResponse.data["initial_response"] || SUCCESS_CONECTION_MESSAGE;
    $("#loading-icon").css("display", "none");
    addMessage(bot_name, initialMessage, false);
    storeMessage(bot_name, initialMessage, false);
  }

  // Manejar el envío del mensaje del usuario
  function handleSendMessage() {
    var userMessage = $("#chat-input").val().trim();
    if (userMessage === "") {
      return;
    }

    addMessage(user_name, userMessage, true);
    storeMessage(user_name, userMessage, true);

    var data = {
      user_input: userMessage,
      env: env,
    };

    sendPostRequest(
      api_base_url + "/secure/api/message",
      data,
      function (botResponse) {
        addMessage(bot_name, botResponse.data.text, false);
        storeMessage(bot_name, botResponse.data.text, false);
      },
      function () {
        showError(SEND_MESSAGE_FAILURE_MESSAGE);
      }
    );

    $("#chat-input").val("");
  }

  // Función para borrar los datos de sesión y reiniciar el chat para este bot
  function resetChat() {
    // Eliminar solo los mensajes, no el ícono de carga
    $("#chat-messages .message").remove();
    // Mostrar el ícono de carga
    $("#loading-icon").css("display", "block");

    sessionStorage.removeItem(tokenKey);
    sessionStorage.removeItem(messagesKey);
    token = "";
    messages = [];
    initializeChat();
  }

  // Función para enviar solicitudes POST
  function sendPostRequest(url, data, onSuccess, onError) {
    $.ajax({
      url: url,
      type: "POST",
      contentType: "application/json",
      headers: {
        Authorization: "Bearer " + token,
      },
      data: JSON.stringify(data),
      success: function (response) {
        if (onSuccess) onSuccess(response);
      },
      error: function () {
        if (onError) onError();
      },
    });
  }

  // Función para agregar un mensaje al chat
  function addMessage(sender, content, isUser) {
    var messageDiv = $("<div></div>")
      .addClass("message p-2 mb-2 rounded")
      .html("<strong>" + sender + ":</strong> " + content);

    if (isUser) {
      messageDiv.addClass("bg-primary text-white");
    } else {
      messageDiv.addClass("bg-light text-dark");
    }

    $("#chat-messages").append(messageDiv);
    $("#chat-messages").scrollTop($("#chat-messages")[0].scrollHeight);
  }

  // Función para guardar mensajes en sessionStorage
  function storeMessage(sender, content, isUser) {
    messages.push({ sender: sender, content: content, isUser: isUser });
    sessionStorage.setItem(messagesKey, JSON.stringify(messages));
  }

  // Función para mostrar errores en el chat
  function showError(errorMessage) {
    addMessage(bot_name, errorMessage, false);
  }

  // Función para convertir un string en hexadecimal
  function stringToHex(str) {
    let hex = "";
    for (let i = 0; i < str.length; i++) {
      hex += str.charCodeAt(i).toString(16);
    }
    return hex;
  }

  // Inicializar el chat al cargar la página
  initialize();
});
